@@ -18,7 +18,7 @@
 #include "ruby.h"
 
 
-#define TMAIL_VERSION "0.9.5"
+#define TMAIL_VERSION "0.9.6"
 
 static VALUE MailScanner;
 static VALUE ScanError;
@@ -197,14 +197,14 @@ static VALUE
 scan_atom(sc)
     struct mails *sc;
 {
-    scan_word(sc, ATOM_SPECIAL);
+    return scan_word(sc, ATOM_SPECIAL);
 }
 
 static VALUE
 scan_token(sc)
     struct mails *sc;
 {
-    scan_word(sc, TOKEN_SPECIAL);
+    return scan_word(sc, TOKEN_SPECIAL);
 }
 
 
@@ -283,6 +283,7 @@ scan_boundary(sc)
 #endif
 
 
+#if 0
 static void
 fwd_eol(sc)
     struct mails *sc;
@@ -321,6 +322,7 @@ bufadd(buf, beg, len, ret, p_var)
         *p_var += len;
     }
 }
+#endif
 
 static VALUE
 scan_comment(sc)
