@@ -15,9 +15,15 @@
 #   end
 #
 
+if RUBY_VERSION < '1.7' then
+  TIMES_MODULE = Time
+else
+  TIMES_MODULE = Process
+end
+
 def benchmark( label, times, &block )
-  begtime = Process.times.utime
+  begtime = TIMES_MODULE.times.utime
   times.times( &block )
-  endtime = Process.times.utime
+  endtime = TIMES_MODULE.times.utime
   $stderr.printf "%-10s: %f sec\n", label, endtime - begtime
 end
