@@ -1,7 +1,7 @@
 #
 # field.rb
 #
-#   Copyright (c) 1998-2000 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
+#   Copyright (c) 1998-2001 Minero Aoki <aamine@loveruby.net>
 #
 #   This program is free software.
 #   You can distribute/modify this program under the terms of
@@ -17,7 +17,7 @@ require 'tmail/encode'
 
 module TMail
 
-  MSGID = /<[^\@>]+\@[^>\@]+>/o
+  MSGID = /<[^\@>]+\@[^>\@]+>/
 
   ZONESTR_TABLE = {
     'jst' =>   9 * 60,
@@ -65,14 +65,6 @@ module TMail
     'z'   =>   0 * 60
   }
 
-  tspecial     = Regexp.quote( '()<>@,.;:\\"/[]?=' )
-  TSPECIAL     = /[#{tspecial}]/
-  control      = '\x00-\x1f\x7f-\xff'
-  CONTROL      = /[#{control}]/
-  INSECURE     = /[#{tspecial}#{control} ]/
-  INSECURE_PH  = /[#{tspecial}#{control}]/
-
-  ENCODED = /=\?[^\s?=]+\?[qb]\?[^\s?=]+\?=/i
 
   class << self
 
@@ -86,69 +78,26 @@ module TMail
         m[1] == '-' ? -sec : sec
       else
         unless min = ZONESTR_TABLE[ str.downcase ] then
-          raise ParseError, "wrong timezone format '#{str}'"
+          raise MailSyntaxError, "wrong timezone format '#{str}'"
         end
         min * 60
       end
     end
 
-    def time2str( time )
-      ret = time.strftime( "%a, #{time.mday} %b %Y %X " )
+    WDAY = %w( Sun Mon Tue Wed Thu Fri Sat wdaybug )
+    MONTH = %w( mbug Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec mbug )
 
-      # Time#gmtime changes self!!!
-      tm = Time.at( time.to_i )
+    def time2str( tm )
       # [ruby-list:7928]
-      offset = tm.to_i - Time.mktime( *tm.gmtime.to_a[0, 6].reverse ).to_i
-      ret << '%+.2d%.2d' % (offset / 60).divmod( 60 )
+      gmt = Time.at(tm.to_i)
+      gmt.gmtime
+      offset = tm.to_i - Time.local( *gmt.to_a[0,6].reverse ).to_i
 
-      ret
-    end
-
-    def quote( str )
-      if INSECURE === str then
-        '"' + str.gsub('"', '\\\\"') << '"'
-      else
-        str
-      end
-    end
-
-    def quote_phrase( str )
-      if INSECURE_PH === str then
-        '"' + str.gsub('"', '\\\\"') << '"'
-      else
-        str
-      end
-    end
-
-    def encoded?( str )
-      ENCODED === str
-    end
-
-
-    # internal use
-
-    def define_header_to_s( mod, use_limit )
-      mod.module_eval %^
-        def encoded( eol = "\r\n", charset = 'j',
-                     ret = ''#{use_limit ? ', limit = 72' : ''} )
-          en = ::TMail::HFencoder.new( ret, eol, charset,
-                  #{use_limit ? 'limit' : 'nil'}  )
-          accept en
-          en.write_in
-          ret
-        end
-
-        def decoded( eol = "\n", charset = 'e',
-                     ret = ''#{use_limit ? ', limit = 72' : ''} )
-          de = ::TMail::HFdecoder.new( ret, charset )
-          accept de
-          de.write_in
-          ret
-        end
-
-        alias to_s encoded
-        alias inspect decoded
-      ^
+      # DO NOT USE strftime: setlocale() breaks it
+      sprintf '%s, %s %s %d %02d:%02d:%02d %+.2d%.2d',
+              WDAY[tm.wday], tm.mday, MONTH[tm.month],
+              tm.year, tm.hour, tm.min, tm.sec,
+              *(offset / 60).divmod(60)
     end
 
   end   # class << self
@@ -156,19 +105,6 @@ module TMail
 
   class HeaderField
 
-    def initialize( fname, fbody, strict )
-      @name = fname
-      fbody.strip!
-      @body = fbody
-      @strict = strict
-
-      @illegal = false
-      @parsed = false
-      @parsing = false
-      # @sync = Sync.new
-    end
-
-
     R     = (1 << 0)  # readable
     W     = (1 << 1)  # writable
     NODUP = (1 << 2)  # dup?
@@ -289,6 +225,22 @@ module TMail
     end
 
 
+    def initialize( fname, fbody, strict )
+      @name = fname
+      fbody.strip!
+      @body = fbody
+      @strict = strict
+
+      @illegal = false
+      @parsed = false
+      @parsing = false
+      # @sync = Sync.new
+    end
+
+    def inspect
+      "#<#{type} #@body>"
+    end
+
     def hash
       @name.hash
     end
@@ -316,9 +268,10 @@ module TMail
 
     def body
       ensure_parsed
-      v = HFdecoder.new( '' )
+      v = HFdecoder.new( s = '' )
       do_accept v
-      v.write_in
+      v.terminate
+      s
     end
 
     def body=( str )
@@ -326,14 +279,16 @@ module TMail
       clear_parse_status
     end
 
-    def accept( strategy )
+
+    include StrategyInterface
+
+    def accept( strategy, dummy = nil )
       ensure_parsed
       strategy.header_name @name
       do_accept strategy
+      strategy.terminate
     end
 
-    ::TMail.define_header_to_s self, true
-
 
     private
 
@@ -424,8 +379,8 @@ module TMail
 
       begin
         init_real
-        Mailp.parse( @body, self, nil )
-      rescue ParseError, ScanError
+        Parser.parse( @body, self, nil )
+      rescue MailSyntaxError, ScanError
         if not pre and ::TMail.encoded?( @body ) then
           pre = @body
           @body = HFdecoder.decode( pre )
@@ -462,10 +417,7 @@ module TMail
 
     parse_on_rw
 
-    header_attr :date, Time, R | W | NODUP, nil, <<S
-      arg = Time.at( arg.to_i )
-      arg.localtime
-S
+    header_attr :date, Time, R | W | NODUP
 
     redefine_eql
     empty_able
@@ -507,16 +459,7 @@ S
     def do_accept( stra )
       stra.meta '<'
       unless route.empty? then
-        first = true
-        @route.each do |i|
-          s = '@' + i
-          if first then
-            first = false
-          else
-            s << ','
-          end
-          stra.meta
-        end
+        stra.meta @route.collect {|i| '@' + i }.join(',')
         stra.meta ':'
       end
       stra.meta @addr
@@ -545,6 +488,10 @@ S
     attr_writer :route   # internal use only
 
 
+    def inspect
+      "#<#{type} #{address}>"
+    end
+
     def Address.join( arr )
       arr.collect {|i| ::TMail.quote i }.join('.')
     end
@@ -588,6 +535,7 @@ S
       route     ==  other.route  and
       phrase    ==  other.phrase
     end
+
     alias == eql?
 
     def dup
@@ -598,12 +546,15 @@ S
       n
     end
 
-    def accept( stra )
+
+    include StrategyInterface
+
+    def accept( strategy, dummy = nil )
       spec_p = !@phrase and @route.empty?
 
       if @phrase then
-        stra.phrase @phrase
-        stra.spc
+        strategy.phrase @phrase
+        strategy.space
       end
       tmp = spec_p ? '' : '<'
       unless @route.empty? then
@@ -611,32 +562,31 @@ S
       end
       tmp << address
       tmp << '>' unless spec_p
-      stra.meta tmp
+      strategy.meta tmp
+      strategy.lwsp ''
     end
 
-    ::TMail.define_header_to_s self, false
-
   end
 
 
 
-  class AddressGroup < DelegateClass( Array )
+  class AddressGroup < DelegateClass(Array)
 
     def initialize( name, arg = nil )
       @name = name
       super arg ? arg.dup : []
     end
 
-
     attr :name, true
     
-    def eql?( oth )
-      super( oth ) and self.name == oth.name
+    def eql?( other )
+      super other and self.name == other.name
     end
+
     alias == eql?
 
     def each_address
-      bare_each do |mbox|
+      each do |mbox|
         if AddressGroup === mbox then
           mbox.each {|i| yield i }
         else
@@ -645,25 +595,26 @@ S
       end
     end
 
-    def accept( stra )
-      stra.phrase @name
-      stra.meta ':'
-      stra.spc
+    include StrategyInterface
+
+    def accept( strategy, dummy = nil )
+      strategy.phrase @name
+      strategy.meta ':'
+      strategy.space
       first = true
-      bare_each do |mbox|
+      each do |mbox|
         if first then
           first = false
         else
-          stra.meta ','
+          strategy.meta ','
         end
-        stra.spc
-        mbox.accept stra
+        strategy.space
+        mbox.accept strategy
       end
-      stra.meta ';'
+      strategy.meta ';'
+      strategy.lwsp ''
     end
 
-    ::TMail.define_header_to_s self, false
-
   end
 
 
@@ -689,7 +640,7 @@ S
       @addr.accept stra
       unless @comments.empty? then
         @comments.each do |c|
-          stra.spc
+          stra.space
           stra.meta '('
           stra.text c
           stra.meta ')'
@@ -728,13 +679,13 @@ S
           first = false
         else
           stra.meta ','
-          stra.spc
+          stra.space
         end
         a.accept stra
       end
       unless @comments.empty? then
         @comments.each do |c|
-          stra.spc
+          stra.space
           stra.meta '('
           stra.text c
           stra.meta ')'
@@ -772,12 +723,24 @@ S
       end
     end
 
+    def msgids
+      ret = []
+      each_msgid {|i| ret.push i }
+      ret
+    end
+
     def each_phrase
       refs.each do |i|
         yield i unless ::TMail.msgid? i
       end
     end
 
+    def phrases
+      ret = []
+      each_phrase {|i| ret.push i }
+      ret
+    end
+
 
     private
 
@@ -791,7 +754,7 @@ S
         if first then
           first = false
         else
-          stra.spc
+          stra.space
         end
         if ::TMail.msgid? i then
           stra.meta i
@@ -847,8 +810,8 @@ S
     header_attr :by,    String, R | W
     header_attr :via,   String, R | W
     header_attr :with,  Array,  R | NODUP
-    header_attr :msgid, String, R | W
-    header_attr :for_,  String, R | W,        [:ford, :for_domain]
+    header_attr :_id,   String, R | W,        [:msgid]
+    header_attr :_for,  String, R | W,        [:for_]
     header_attr :date,  Time,   R | W | NODUP
 
     redefine_eql
@@ -858,7 +821,7 @@ S
     private
 
     def init
-      @from = @by = @via = @with = @msgid = @for_ = nil
+      @from = @by = @via = @with = @_id = @_for = nil
       @with = []
       @date = nil
     end
@@ -866,18 +829,18 @@ S
     Tag = %w( from by via with id for )
 
     def do_accept( stra )
-      val = [ @from, @by, @via ] + @with + [ @msgid, @for_ ]
+      val = [ @from, @by, @via ] + @with + [ @_id, @_for ]
       val.compact!
       c   = [ @from ? 1 : 0, @by ? 1 : 0, @via ? 1 : 0,
-              @with.size, @msgid ? 1 : 0, @for_ ? 1 : 0 ]
+              @with.size, @_id ? 1 : 0, @_for ? 1 : 0 ]
 
       i = 0
       c.each_with_index do |count, idx|
         label = Tag[idx]
         count.times do
-          stra.spc unless i == 0
+          stra.space unless i == 0
           stra.meta label
-          stra.spc
+          stra.space
           stra.meta val[i]
           i += 1
         end
@@ -885,7 +848,7 @@ S
 
       stra.meta ';'
       if @date then
-        stra.spc
+        stra.space
         stra.meta ::TMail.time2str( @date )
       end
     end
@@ -946,7 +909,7 @@ S
     def do_accept( stra )
       if @key then
         stra.meta @encrypter + ','
-        stra.spc
+        stra.space
         stra.meta @keyword
       else
         stra.meta @encrypter
@@ -1017,10 +980,8 @@ S
       stra.meta "#{@main}/#{@sub}"
       @params.each do |k,v|
         stra.meta ';'
-        stra.spc
-        stra.meta k
-        stra.meta '='
-        stra.value v
+        stra.space
+        stra.kv_pair k, v
       end
     end
 
@@ -1080,10 +1041,8 @@ S
       stra.meta @disposition
       @params.each do |k,v|
         stra.meta ';'
-        stra.spc
-        stra.meta k
-        stra.meta '='
-        stra.value v
+        stra.space
+        stra.kv_pair k, v
       end
     end
       
